'''
This module provides a convenience class for a vtk scene widget. Mouse
interaction is enabled:

    interaction:
    left + move:    rotate
    middle + move:  translate
    right + move:   zoom
    wheel:          zoom

    picking:
    left click on actor:  pick (select) actor
    left click anywhere:  deselect actor
    right click:          context menu request
'''


from vtk.qt4.QVTKRenderWindowInteractor import QVTKRenderWindowInteractor

from PyQt4.Qt import *
from PyQt4.QtCore import *

import vtk




class VtkWidget(QVTKRenderWindowInteractor):
    def __init__(self, parent=None):
        QVTKRenderWindowInteractor.__init__(self, parent)

        # init interaction helper variables
        self.mouseMoved = 0
        self.pickedActor = None

        # renderer, render window, camera
        self.renderer = vtk.vtkRenderer()
        self.renderWindow = self.GetRenderWindow()
        self.renderWindow.AddRenderer(self.renderer)
        self.camera = self.renderer.GetActiveCamera()

        # picker and interaction style
        self.picker = vtk.vtkPicker()
        self.pickStyle = vtk.vtkInteractorStyleTrackballCamera()

        self.pickStyle.AddObserver('LeftButtonPressEvent',
                                    self.onLeftButtonDown)
        self.pickStyle.AddObserver('LeftButtonReleaseEvent',
                                    self.onLeftButtonRelease)

        self.pickStyle.AddObserver('RightButtonPressEvent',
                                    self.onRightButtonDown)
        self.pickStyle.AddObserver('RightButtonReleaseEvent',
                                    self.onRightButtonRelease)

        self.pickStyle.AddObserver('MouseMoveEvent', self.onMouseMove)

        self.SetInteractorStyle(self.pickStyle)

        # set background and go for it
        self.setBackground((0.5, 0.5, 0.5), (0.9, 0.9, 0.9))
        self.Initialize()
        self.Start()

    def onLeftButtonDown(self, renWin, event):
        self.mouseMoved = 0
        self.pickStyle.OnLeftButtonDown()

    def onRightButtonDown(self, renWin, event):
        self.mouseMoved = 0
        self.pickStyle.OnRightButtonDown()


    def onLeftButtonRelease(self, renWin, event):
        if self.mouseMoved == 0:
            x, y = self._Iren.GetEventPosition()
            self.picker.Pick([x, y, 0], self.renderer)
            actor =  self.picker.GetActor()
            self.onLeftClick(actor)
        self.pickStyle.OnLeftButtonUp()

    def onRightButtonRelease(self, renWin, event):
        if self.mouseMoved == 0:
            x, y = self._Iren.GetEventPosition()
            self.picker.Pick([x, y, 0], self.renderer)
            actor =  self.picker.GetActor()
            self.onRightClick(actor)
        self.pickStyle.OnRightButtonUp()


    def onMouseMove(self, renWin, event):
        self.mouseMoved = 1
        self.pickStyle.OnMouseMove()


    def onLeftClick(self, actor):
        '''Callback for mouse pick event.'''
        if actor is not None:
            #actor.GetProperty().SetOpacity(0.5)
            actor.GetProperty().SetColor(1,0,0)
            self.pickedActor = actor
        else:
            if self.pickedActor is not None:
                #self.pickedActor.GetProperty().SetOpacity(1)
                self.pickedActor.GetProperty().SetColor(1,1,1)

    def onRightClick(self, actor):
        '''
        Callback for context menu request - the actor should be
        selected/picked too!
        '''
        print 'context menu request'


    def addActor(self, actor):
        self.renderer.AddActor(actor)


    def setBackground(self, firstColor, secondColor=None):
        self.renderer.SetBackground(firstColor)
        self.renderer.GradientBackgroundOff()
        if secondColor:
            self.renderer.SetBackground2(secondColor)
            self.renderer.GradientBackgroundOn()


    def setAAFrames(self, n):
        if isinstance(n, int) and n >= 0:
            self._RenderWindow.SetAAFrames(n)







if __name__ == '__main__':
    class Gui(QMainWindow):
        def __init__(self):
            QMainWindow.__init__(self)

            # create widgets and layout
            self.top= QWidget()
            self.topBox= QHBoxLayout(self.top)

            self.cw = QWidget()
            self.hBox = QHBoxLayout(self.cw)
            self.hBox.setMargin(0)

            self.vtkWidget = VtkWidget(self.cw)
            self.hBox.addWidget(self.vtkWidget)

            self.topBox.addWidget(self.cw)
            self.setCentralWidget(self.top)

            # create a clinder
            part = vtk.vtkCylinderSource()
            part.SetResolution(20)
            part.SetRadius(1)
            part.SetHeight(10)

            # map geometry to OpenGL data
            mapper = vtk.vtkPolyDataMapper()
            mapper.SetInputConnection(part.GetOutputPort())

            # create actor for moving, rotating, scaling, appearance etc.
            self.actor = vtk.vtkActor()
            self.actor.SetMapper(mapper)
            self.actor.GetProperty().SetOpacity(1)
            self.actor.SetScale(0.1)

            # add actor to widget and render scene
            self.vtkWidget.addActor(self.actor)
            self.vtkWidget.Render()



    app = QApplication(['3'])
    gui = Gui()
    gui.show()
    app.exec_()